#import "FLNativeView.h"
@import DyteSdk;



@implementation FLNativeView {
    DyteMeetingView *_view;
    FlutterMethodChannel* _methodChannel;
}

- (instancetype)initWithFrame:(CGRect)frame
               viewIdentifier:(int64_t)viewId
                    arguments:(id _Nullable)args
                         dyteView:(DyteMeetingView*)view
              binaryMessenger:(NSObject<FlutterBinaryMessenger>*)messenger {
    
    if (self = [super init]) {
        _methodChannel = [FlutterMethodChannel methodChannelWithName:@"dyte_client" binaryMessenger:messenger];
        DyteMeetingConfig *meetingConfig = [[DyteMeetingConfig alloc] init];
        meetingConfig.roomName = [args valueForKey:@"roomName"];
        meetingConfig.authToken =[args valueForKey:@"authToken"];
        meetingConfig.apiBase = [args valueForKey:@"apiBase"];
        meetingConfig.orgId = [args valueForKey:@"orgId"];
        meetingConfig.showSetupScreen = [args[@"showSetupScreen"] boolValue];
        meetingConfig.autoTune =  [args[@"autoTune"] boolValue];
        meetingConfig.uiConfig = [args valueForKey:@"uiConfig"];
        _view = [view initWithFrame: CGRectMake(0, 0,  [[args valueForKey:@"width"] doubleValue],   [[args valueForKey:@"height"] doubleValue])];
        _view.delegate = self;
        [_view join:meetingConfig];
    }
    return self;
}

- (void)meetingJoined {
    [_methodChannel invokeMethod:@"eventMeetingJoined" arguments:NULL];
}

- (void)meetingEnded {
    [_methodChannel invokeMethod:@"eventMeetingEnded" arguments:NULL];
}

- (void)meetingDisconnect {
    [_methodChannel invokeMethod:@"eventMeetingDisconnected" arguments:NULL];
}

- (void)meetingConnected {
    [_methodChannel invokeMethod:@"eventMeetingConnected" arguments:NULL];
}

- (void)localMediaConnected {
     [_methodChannel invokeMethod:@"eventLocalMediaConnected" arguments:NULL];
}
//
- (void)participantLeave:(DyteParticipant*)participant  {
    NSDictionary *properties =   @{
            @"name" : participant.name,
            @"id": participant.id,
            @"picture": participant.picture ? participant.picture : @"",
            @"audioEnabled": @(participant.audioEnabled),
            @"videoEnabled": @(participant.videoEnabled),
            @"isPinned": @(participant.isPinned),
            @"clientSpecificId": participant.clientSpecificId
    };
    [_methodChannel invokeMethod:@"eventParticipantLeave" arguments:properties];
}
- (void)participantJoin:(DyteParticipant*)participant {
    NSDictionary *properties =   @{
            @"name" : participant.name ?  participant.name : @"",
            @"id": participant.id ? participant.id : @"",
            @"picture": participant.picture ? participant.picture : @"",
            @"audioEnabled": @(participant.audioEnabled),
            @"videoEnabled": @(participant.videoEnabled),
            @"isPinned": @(participant.isPinned),
            @"clientSpecificId": participant.clientSpecificId ?  participant.clientSpecificId : @""
    };
    [_methodChannel invokeMethod:@"eventParticipantJoin" arguments:properties];
}
- (void)participantUpdate:(DyteParticipant*)participant {
    NSDictionary *properties =   @{
            @"name" : participant.name ?  participant.name : @"",
            @"id": participant.id ? participant.id : @"",
            @"picture": participant.picture ? participant.picture : @"",
            @"audioEnabled": @(participant.audioEnabled),
            @"videoEnabled": @(participant.videoEnabled),
            @"isPinned": @(participant.isPinned)
    };
    [_methodChannel invokeMethod:@"eventParticipantUpdate" arguments:properties];
}

- (DyteMeetingView*)view {
    return _view;
}

@end
