import 'dart:async';
import 'dart:convert';
import 'dyteParticipant.dart';
import 'dyteSelfParticipant.dart';
import 'package:flutter/services.dart';
import 'package:eventify/eventify.dart';

class DyteMeetingHandler {
  static const MethodChannel _channel = const MethodChannel('dyte_client');
  late EventEmitter events;

  DyteMeetingHandler(){
    this.events = new EventEmitter();
    _channel.setMethodCallHandler(_methodCallHandler);
  }

  Future<void> _methodCallHandler(MethodCall call) async {
    switch (call.method) {
      case 'eventMeetingJoined' :
        events.emit('meetingJoin',null,null);
        break;
      case 'eventMeetingEnded' :
        events.emit('meetingEnd',null,null);
        break;
      case 'eventMeetingDisconnected' :
        events.emit('meetingDisconnected',null,null);
        break;
      case 'eventMeetingConnected' :
        events.emit('meetingConnected',null,null);
        break;  
      case 'eventParticipantLeave':
        final Map ret = call.arguments;
        events.emit('participantLeave',null, new DyteParticipant(ret));
        break;
      case 'eventParticipantJoin':
        final Map ret = call.arguments;
        events.emit('participantJoin',null, new DyteParticipant(ret));
        break;
      case 'eventLocalMediaConnected': 
         events.emit('localMediaConnected',null,null);
        break;
      default:
        break;
    }
  }

  Future<List<DyteParticipant>> get participants async {
    final List<Map<dynamic, dynamic>> ret =
        await _channel.invokeMethod('getMeetingParticipants');
    List<DyteParticipant> participants = <DyteParticipant>[];
    ret.forEach((p) {
      participants.add(new DyteParticipant(p));
    });
    return participants;
  }

  Future<DyteSelfParticipant> get self async {
    final Map<dynamic, dynamic> ret = await _channel.invokeMethod('getSelf');
    return new DyteSelfParticipant(ret);
  }

  void updateUIConfig(Map<String, dynamic> config){
       _channel.invokeMethod('updateUIConfig', {"config": jsonEncode(config)});
  }
}
